# -*- coding: UTF-8 -*-
import os
import re
import sys
import json
import pwd
import grp
import socket
import getpass

if sys.version_info < (3, ):
    bytes = str
    PY3 = False
else:
    unicode = str
    PY3 = True


def get_tags():
    return ['1', '1.17']


def run_task(runcontext):
    ansible_params = runcontext.get_mod_params_instance()
    runcontext.set_mod_params_instance(ansible_params, "chdir", None)
    runcontext.set_mod_params_instance(ansible_params, "executable", None)

    module = runcontext.get_module_file("shell.py")
    rc, out = run_audit(runcontext, ansible_params, module)
    return rc, out


def run_audit(runcontext, ansible_params, module):
    curtemplate = runcontext.get_template()
    tmp_vars = {}
    tmp_vars["policyresult"] = "failed"
    tmp_vars["checkresult1"] = "failed"
    tmp_vars["scancommand1"] = r'''grep -vE "^$|\s*#" /etc/ssh/sshd_config | grep Subsystem'''
    tmp_vars["scancommand2"] = r'''matchfind=`grep -Ev "^$|^\s*#" /etc/ssh/sshd_config | grep Match`;if [ -n "$matchfind" ];then grep -Ev "^$|^\s*#" /etc/ssh/sshd_config | sed -rn '/^Match\s+(User|Group)/,$p';fi;'''
    tmp_vars["systemvalue1"] = ""
    ret_code = -1

    runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand1"])
    arc1, astdout1, astderr1 = runcontext.invoke_module(module, ansible_params)
    runcontext.log_debug_invoke_result(arc1, astdout1, astderr1)
    runcontext.sleep()

    if arc1 != 0:
        return (-2, "Failed to invoke module: %s." % astdout1)

    resultjson1 = json.loads(astdout1)
    astdout1 = resultjson1["stdout"]
    astderr1 = resultjson1["stderr"]

    match1_found = re.match("^$", astdout1)
    if match1_found is not None:
        tmp_vars["systemvalue1"] = ''
        tmp_vars["checkresult1"] = "pass"
    else:
        retErrMsg = []
        retCnfMsg = []
        auxList = []

        # get config params
        runcontext.set_mod_params_instance(ansible_params, "command", tmp_vars["scancommand2"])
        arc2, astdout2, astderr2 = runcontext.invoke_module(module, ansible_params)
        runcontext.log_debug_invoke_result(arc2, astdout2, astderr2)
        runcontext.sleep()

        if arc2 != 0:
            return (-2, "Failed to invoke module: %s." % astdout2)
        resultjson2 = json.loads(astdout2)
        astdout2 = resultjson2["stdout"]
        astderr2 = resultjson2["stderr"]

        params = [x.strip() for x in astdout2.replace('\t', '    ').strip().split('\n')]
        p_match = params[0]

        if 'user' in p_match.lower():
            pattern = re.compile(r'(\w+)\s+(\w+)\s+([\w._][\w.-_@$]*)')
            m = pattern.findall(p_match)
            if m > 0:
                p_user = m[0][2]
                shell = pwd.getpwnam(p_user).pw_shell

                if shell not in ['/sbin/nologin', '/bin/false']:
                    retErrMsg.append(p_match + ': %s in /etc/passwd config with %s.(Match User %s)' % (p_user, shell, p_user))
                else:
                    retCnfMsg.append(p_match)
        elif 'group' in p_match.lower():
            pattern = re.compile(r'(\w+)\s+(\w+)\s+([\w._][\w.-_@$]*)')
            m = pattern.findall(p_match)
            if m > 0:
                p_group = m[0][2]
                userList = grp.getgrnam(p_group).gr_mem
                if 0 == len(userList):
                    retCnfMsg.append(p_match)
                for user in userList:
                    shell = pwd.getpwnam(user).pw_shell
                    if shell not in ['/sbin/nologin', '/bin/false']:
                        retErrMsg.append('the %s which in group %s ,config with %s in /etc/passwd.(Match Group %s)' % (user, p_group, shell, p_group))
                    else:
                        retCnfMsg.append(p_match)
        else:
            retErrMsg.append('Match User/Group not configured')

        params.pop(0)

        # params under match deal with
        cerrnum = 0
        ferrnum = 0
        xerrnum = 0
        aerrnum = 0
        paramlist = [x.split(" ")[0] for x in params]
        if len(paramlist) == 0 and astdout2:
            retErrMsg.append('ChrootDirectory/ForceCommand/X11Forwarding/AllowTcpForwarding not configured')
        for param in params:
            if 'chrootdirectory' in param.lower():
                auxList.append('chrootdirectory')
                chroot_path = param.split()[1]
                chroot_path = chroot_path.replace('%h', socket.gethostname()).replace('%u', getpass.getuser())
                chroot_path = runcontext.replace_inject_characters(chroot_path)

                if os.path.exists(chroot_path):
                    if chroot_path not in ['/', '/root', '/etc', '/bin', '/boot', '/lib', '/lib64']:
                        cmds = 't1=$(stat -L --format="%%a" %s);t2=$(stat -L --format="%%U" %s);t3=$(stat -L --format="%%G" %s);if [ 0"$t2" = 0"root" -a 0"$t3" = 0"root" ] &&  (((0$t1 & 0022) == 0));then echo "true:%s  $t2:$t3 $t1";else echo "false:%s $t2:$t3 $t1";fi' % (chroot_path, chroot_path, chroot_path, chroot_path, chroot_path)

                        runcontext.set_mod_params_instance(ansible_params, "command", cmds)
                        arc4, astdout4, astderr4 = runcontext.invoke_module(module, ansible_params)
                        runcontext.log_debug_invoke_result(arc4, astdout4, astderr4)
                        runcontext.sleep()

                        if arc4 != 0:
                            return (-2, "Failed to invoke module: %s." % astdout4)

                        resultjson4 = json.loads(astdout4)
                        astdout4 = resultjson4["stdout"]
                        astderr4 = resultjson4["stderr"]
                        output = astdout4

                        if 'false' in output:
                            retErrMsg.append(''.join([param, ' (', output.strip().lstrip('false:'), ' )']))
                        else:
                            retCnfMsg.append(param)
                    else:
                        retErrMsg.append(param)

                    cmds = 'find %s  -perm /022  2>/dev/null -printf "%%h/%%f    %%m\n"' % chroot_path
                    runcontext.set_mod_params_instance(ansible_params, "command", cmds)
                    arc5, astdout5, astderr5 = runcontext.invoke_module(module, ansible_params)
                    runcontext.log_debug_invoke_result(arc5, astdout5, astderr5)
                    runcontext.sleep()

                    if arc5 != 0:
                        return (-2, "Failed to invoke module: %s." % astdout5)

                    resultjson5 = json.loads(astdout5)
                    astdout5 = resultjson5["stdout"]
                    astderr5 = resultjson5["stderr"]
                    output = astdout5

                    if output != '':
                        retErrMsg.append(output)
            else:
                if 'ChrootDirectory' not in paramlist:
                    if cerrnum == 0:
                        retErrMsg.append('ChrootDirectory not configured')
                        cerrnum += 1

            if 'forcecommand' in param.lower():
                auxList.append('forcecommand')
                if re.match(r'\s*forcecommand\s+internal-sftp\s*', param.lower()):
                    retCnfMsg.append(param)
                else:
                    retErrMsg.append(param)
            else:
                if 'ForceCommand' not in paramlist:
                    if ferrnum == 0:
                        retErrMsg.append('ForceCommand not configured')
                        ferrnum += 1

            if 'x11forwarding' in param.lower():
                auxList.append('x11forwarding')
                if re.match(r'\s*x11forwarding\s+no\s*', param.lower()):
                    retCnfMsg.append(param)
                else:
                    retErrMsg.append(param)
            else:
                if 'X11Forwarding' not in paramlist:
                    if xerrnum == 0:
                        retErrMsg.append('X11Forwarding not configured')
                        xerrnum += 1

            if 'allowtcpforwarding' in param.lower():
                auxList.append('allowtcpforwarding')
                if re.match(r'\s*allowtcpforwarding\s+no\s*', param.lower()):
                    retCnfMsg.append(param)
                else:
                    retErrMsg.append(param)
            else:
                if 'AllowTcpForwarding' not in paramlist:
                    if aerrnum == 0:
                        retErrMsg.append('AllowTcpForwarding not configured')
                        aerrnum += 1

        tmp_vars["systemvalue1"] = '\n'.join(retErrMsg)

        if 0 == len(retErrMsg) and 4 == len(set(auxList)):
            tmp_vars["systemvalue1"] = '\n'.join(retCnfMsg)
            tmp_vars["checkresult1"] = "pass"

    if tmp_vars["checkresult1"] == "pass":
        tmp_vars["policyresult"] = "pass"
        ret_code = 0

    rc, out = curtemplate.render(runcontext.get_template_file('t_1_17.j2'), **tmp_vars)

    if rc == False:
        return (-2, "Failed to render template.%s" % out)

    return ret_code, out
